@@ -1,6 +1,6 @@
 package Getopt::EX::Hashed;
 
-our $VERSION = '0.9905';
+our $VERSION = '0.9906';
 
 =head1 NAME
 
@@ -8,7 +8,7 @@ Getopt::EX::Hashed - Hash store object automation
 
 =head1 VERSION
 
-Version 0.9905
+Version 0.9906
 
 =head1 SYNOPSIS
 
@@ -32,10 +32,10 @@ Version 0.9905
 =head1 DESCRIPTION
 
 B<Getopt::EX::Hashed> is a module to automate a hash object to store
-command line option values.  Major objective of this module is to
-integrate initialization and specification into single place.  Module
-name shares B<Getopt::EX>, but it works independently from other
-modules included in B<Getopt::EX>, so far.
+command line option values.  Major objective of this module is
+integrating initialization and specification into single place.
+Module name shares B<Getopt::EX>, but it works independently from
+other modules included in B<Getopt::EX>, so far.
 
 In the current implementation, using B<Getopt::Long>, or compatible
 module such as B<Getopt::EX::Long> is assumed.  It is configurable,
@@ -51,7 +51,7 @@ Declare option parameters in a form of:
 
 If array reference is given, multiple names can be declared at once.
 
-    has [ 'left', 'right' ] => ( param => value, ... );
+    has [ 'left', 'right' ] => ( spec => "=i" );
 
 If the name start with plus (C<+>), given parameters are added to
 current value.
@@ -73,7 +73,7 @@ Declaration
 
 will be compiled into string:
 
-    start|s|begin:i
+    start|s|begin=i
 
 which conform to C<Getopt::Long> definition.  Of course, you can write
 as this:
@@ -100,7 +100,7 @@ There is no difference with ones in B<spec> parameter.
 =item B<default> => I<value>
 
 Set default value.  If no default is given, the member is initialized
-as C<undef>.  See B<action>.
+as C<undef>.
 
 =item B<action> => I<coderef>
 
@@ -112,17 +112,29 @@ option.  When called, hash object is passed through C<$_>.
         $_->{left} = $_->{right} = $_[1];
     };
 
-You can use this for C<< "<>" >> too.  In this case, spec parameter
-does not matter and is not required.
+You can use this for C<< "<>" >> too.  In that case, spec parameter
+does not matter and not required.
 
     has ARGV => default => [];
     has "<>" => action => sub {
         push @{$_->{ARGV}}, $_[0];
     };
 
-In fact, B<default> and B<action> parameters are twins and works same.
-Parameter B<action> is just a little more understandable, one byte
-shorter, and verifies the value.  They are exclusive.
+In fact, B<default> parameter takes code reference too.  It is stored
+in the hash object and the code works almost same.  But the hash value
+can not be used for option storage.
+
+Because B<action> function intercept the option assignment, it can be
+used to verify the parameter.
+
+    has age =>
+        spec => '=i',
+        action => sub {
+            my($name, $i) = @_;
+            (0 <= $i and $i <= 150) or
+                die "$name: have to be in 0 to 150 range.\n";
+            $_->{$name} = $i;
+        };
 
 =back
 
@@ -147,12 +159,13 @@ appropriate parameters.
 
 is just a shortcut for:
 
-    GetOptions($obj, $obj->optspec)
+    GetOptions($obj->optspec)
 
 =item B<optspec>
 
 Return option specification list which can be given to C<GetOptions>
-function with the hash object.
+function.  GetOptions has a capability of storing values in a hash, by
+giving the hash reference as a first argument, but it is not expected.
 
 =item B<use_keys>
 
@@ -217,10 +230,12 @@ use Data::Dumper;
 use Exporter 'import';
 our @EXPORT = qw(has new);
 
+use List::Util qw(first);
+
 my @Member;
-my %Member;
 
 my %Config = (
+    DEBUG_PRINT        => 0,
     LOCK_KEYS          => 1,
     REPLACE_UNDERSCORE => 1,
     RESET_AFTER_NEW    => 0,
@@ -243,22 +258,22 @@ sub unimport {
 }
 
 sub reset {
-    @Member = %Member = ();
+    @Member = ();
     return $_[0];
 }
 
 sub has {
-    my($name, %param) = @_;
-    my @name = ref $name eq 'ARRAY' ? @$name : $name;
-    for (@name) {
-	my $append = s/^\+//;
+    my($key, @param) = @_;
+    my @name = ref $key eq 'ARRAY' ? @$key : $key;
+    for my $name (@name) {
+	my $append = $name =~ s/^\+//;
+	my $i = first { $Member[$_]->[0] eq $name } 0 .. $#Member;
 	if ($append) {
-	    $Member{$_} or die "$_: Not defined\n";
-	    $Member{$_} = { %{$Member{$_}}, %param };
+	    defined $i or die "$name: Not found\n";
+	    push @{$Member[$i]}, @param;
 	} else {
-	    $Member{$_} and die "$_: Duplicated\n";
-	    push @Member, $_;
-	    $Member{$_} = \%param;
+	    defined $i and die "$name: Duplicated\n";
+	    push @Member, [ $name, @param ];
 	}
     }
 }
@@ -266,28 +281,17 @@ sub has {
 sub new {
     my $class = shift;
     my $obj = bless {}, __PACKAGE__;
-    my $member = $obj->{__Hash__} = { %Member };
-    my $order = $obj->{__Order__} = [ @Member ];
     our @ISA = $class if $class ne __PACKAGE__;
+    my $member = $obj->{__Hash__} = {
+	map {
+	    my($key, %param) = @$_;
+	    $key => \%param;
+	} @Member
+    };
+    my $order = $obj->{__Order__} = [ map $_->[0], @Member ];
     for my $key (@{$order}) {
 	my $m = $member->{$key};
-	$obj->{$key} = do {
-	    my($default, $action) = @{$m}{qw(default action)};
-	    if ($action) {
-		defined $default and
-		    die "$key: Don't define both default and action.\n";
-		ref($action) eq 'CODE' or
-		    die "$key: action have to be coderef.\n";
-		$default = $action;
-	    }
-	    if (defined($default) and ref($default) eq 'CODE') {
-		sub {
-		    $default->(@_) for $obj;
-		};
-	    } else {
-		$default;
-	    }
-	};
+	$obj->{$key} = $m->{default};
     }
     lock_keys %{$obj} if $Config{LOCK_KEYS};
     __PACKAGE__->reset if $Config{RESET_AFTER_NEW};
@@ -297,26 +301,48 @@ sub new {
 sub optspec {
     my $obj = shift;
     my $member = $obj->{__Hash__};
-    do {
-	map  { _optspec($obj, @$_) }
+    my @optlist = do {
+	map  {
+	    my($name, $spec) = @$_;
+	    my $compiled = _compile($obj, $name, $spec);
+	    my $m = $member->{$name};
+	    my $dest = do {
+		if (my $action = $m->{action}) {
+		    ref $action eq 'CODE' or
+			die "$name: action must be coderef.\n";
+		    sub { &$action for $obj };
+		} else {
+		    if (ref $obj->{$name} eq 'CODE') {
+			sub { &{$obj->{$name}} for $obj };
+		    } else {
+			\$obj->{$name};
+		    }
+		}
+	    };
+	    $compiled => $dest;
+	}
+	# spec .= alias
 	map  {
 	    if (my $alias = $member->{$_->[0]}->{alias}) {
 		$_->[1] .= " $alias";
 	    }
 	    $_;
 	}
+	# spec = '' if $name eq = '<>'
 	grep {
 	    $_->[0] eq '<>' and $_->[1] //= '';
 	    defined $_->[1];
 	}
+	# get spec
 	map  { [ $_ => $member->{$_}->{spec} ] }
 	@{$obj->{__Order__}};
     };
+    @optlist;
 }
 
 my $spec_re = qr/[!+=:]/;
 
-sub _optspec {
+sub _compile {
     my $obj = shift;
     my($name, $args) = @_;
 
@@ -344,7 +370,7 @@ sub getopt {
     my $obj = shift;
     my $getopt = caller . "::" . $Config{GETOPT};
     no strict 'refs';
-    &{$getopt}($obj, $obj->optspec);
+    &{$getopt}($obj->optspec);
 }
 
 sub use_keys {
